@@ -172,10 +172,10 @@ class (
 
 data ARule m = forall key value . Rule key value => ARule (key -> Maybe (m value))
 
-ruleKey :: Rule key value => (key -> Maybe (m value)) -> key
+ruleKey :: (key -> Maybe (m value)) -> key
 ruleKey = err "ruleKey"
 
-ruleValue :: Rule key value => (key -> Maybe (m value)) -> value
+ruleValue :: (key -> Maybe (m value)) -> value
 ruleValue = err "ruleValue"
 
 
@@ -404,17 +404,13 @@ run opts@ShakeOptions{..} rs = (if shakeLineBuffering then lineBuffering else id
     let diagnostic = if shakeVerbosity >= Diagnostic then outputLocked Diagnostic . ("% "++) else const $ return ()
     let output v = outputLocked v . abbreviate shakeAbbreviations
 
-    except <- newIORef (Nothing :: Maybe (String, SomeException))
-    let staunch act | not shakeStaunch = void act
-                    | otherwise = do
-            res <- try act
-            case res of
-                Left err -> do
-                    let named = maybe "" (abbreviate shakeAbbreviations . shakeExceptionTarget) . fromException
-                    atomicModifyIORef except $ \v -> (Just $ fromMaybe (named err, err) v, ())
-                    let msg = show err ++ "Continuing due to staunch mode, this error will be repeated later"
-                    when (shakeVerbosity >= Quiet) $ output Quiet msg
-                Right _ -> return ()
+    except <- newIORef (Nothing :: Maybe (String, ShakeException))
+    let raiseError err
+            | not shakeStaunch = throwIO err
+            | otherwise = do
+                let named = abbreviate shakeAbbreviations . shakeExceptionTarget
+                atomicModifyIORef except $ \v -> (Just $ fromMaybe (named err, err) v, ())
+                -- no need to print exceptions here, they get printed when they are wrapped
 
     lint <- if isNothing shakeLint then return $ const $ return () else do
         dir <- getCurrentDirectory
@@ -451,7 +447,10 @@ run opts@ShakeOptions{..} rs = (if shakeLineBuffering then lineBuffering else id
                     let s0 = Global database pool cleanup start ruleinfo output opts diagnostic lint after absent
                     let s1 = Local emptyStack shakeVerbosity Nothing [] 0 [] [] []
                     forM_ (actions rs) $ \act -> do
-                        addPool pool $ runAction s0 s1 act $ \x -> staunch $ either throwIO return x
+                        addPool pool $ runAction s0 s1 act $ \x -> case x of
+                            Left e -> raiseError =<< shakeException s0 (return ["Top-level action/want"]) e
+                            Right x -> return x
+                maybe (return ()) (throwIO . snd) =<< readIORef except
 
                 when (null $ actions rs) $ do
                     when (shakeVerbosity >= Normal) $ output Normal "Warning: No want/action statements, nothing to do"
@@ -476,7 +475,6 @@ run opts@ShakeOptions{..} rs = (if shakeLineBuffering then lineBuffering else id
                         when (shakeVerbosity >= Normal) $
                             output Normal $ "Writing live list to " ++ file
                         (if file == "-" then putStr else writeFile file) $ unlines liveFiles
-            maybe (return ()) (throwIO . snd) =<< readIORef except
             sequence_ . reverse =<< readIORef after
 
 
@@ -496,15 +494,6 @@ abbreviate abbrev = f
         f (x:xs) = x : f xs
 
 
-wrapStack :: IO [String] -> IO a -> IO a
-wrapStack stk act = catch_ act $ \(SomeException e) -> case cast e of
-    Just s@ShakeException{} -> throwIO s
-    Nothing -> do
-        stk <- stk
-        if null stk then throwIO e
-         else throwIO $ ShakeException (last stk) stk $ SomeException e
-
-
 runAction :: Global -> Local -> Action a -> Capture (Either SomeException a)
 runAction g l (Action x) k = runRAW g l x k
 
@@ -552,7 +541,7 @@ applyKeyValue ks = do
                 when (shakeLint globalOptions == Just LintTracker)
                     trackCheckUsed
                 Action $ fmap ((,) res) getRW) $ \x -> case x of
-                    Left e -> (continue =<<) $ try $ wrapStack (showStack globalDatabase stack) $ throwIO e
+                    Left e -> continue . Left . toException =<< shakeException global (showStack globalDatabase stack) e
                     Right (res, Local{..}) -> do
                         dur <- time
                         globalLint $ "after building " ++ top
@@ -565,6 +554,20 @@ applyKeyValue ks = do
     return vs
 
 
+-- | Turn a normal exception into a ShakeException, giving it a stack and printing it out if in staunch mode.
+--   If the exception is already a ShakeException (e.g. it's a child of ours who failed and we are rethrowing)
+--   then do nothing with it.
+shakeException :: Global -> IO [String] -> SomeException -> IO ShakeException
+shakeException Global{globalOptions=ShakeOptions{..},..} stk e@(SomeException inner) = case cast inner of
+    Just e@ShakeException{} -> return e
+    Nothing -> do
+        stk <- stk
+        e <- return $ ShakeException (last $ "Unknown call stack" : stk) stk e
+        when (shakeStaunch && shakeVerbosity >= Quiet) $
+            globalOutput Quiet $ show e ++ "Continuing due to staunch mode"
+        return e
+
+
 -- | Apply a single rule, equivalent to calling 'apply' with a singleton list. Where possible,
 --   use 'apply' to allow parallelism.
 apply1 :: Rule key value => key -> Action value
@@ -815,10 +818,13 @@ withResource :: Resource -> Int -> Action a -> Action a
 withResource r i act = do
     Global{..} <- Action getRO
     liftIO $ globalDiagnostic $ show r ++ " waiting to acquire " ++ show i
-    Action $ captureRAW $ \continue -> acquireResource r globalPool i $ do
-        globalDiagnostic $ show r ++ " acquired " ++ show i
-        continue $ Right ()
-    res <- Action $ tryRAW $ fromAction $ blockApply ("Within withResource using " ++ show r) act
+    offset <- liftIO $ offsetTime
+    Action $ captureRAW $ \continue -> acquireResource r globalPool i $ continue $ Right ()
+    res <- Action $ tryRAW $ fromAction $ blockApply ("Within withResource using " ++ show r) $ do
+        offset <- liftIO offset
+        liftIO $ globalDiagnostic $ show r ++ " acquired " ++ show i ++ " in " ++ showDuration offset
+        Action $ modifyRW $ \s -> s{localDiscount = localDiscount s + offset}
+        act
     liftIO $ releaseResource r globalPool i
     liftIO $ globalDiagnostic $ show r ++ " released " ++ show i
     Action $ either throwRAW return res
